// ========================================================================================
//    BJ Printer Setup Utility Program
//    DOS Base
//    BJ Printer (such as the A250 and A5311)
//
//    1. Version 1.00 [August 18, '95]       : Tsukasa Jackie AIBA    : create new
//    2. Version 1.01 [September 21, '95]    : Tsukasa Jackie AIBA    : add A240
//    3. Version 1.02 [September 26, '95]    : Tsukasa Jackie AIBA    : mono display
//    4. Version 1.10 [April 30, '96]        : Tsukasa Jackie AIBA    : Added BJC-80/240/4200/4550
//
//    Copyright Canon Information Systems, Inc. 1995
// ========================================================================================

// *********************************************************************************
//        define BJL(E) commands for A250
// *********************************************************************************

//             Define                        Command,                          Number of Character, Parameter, NV-RAM

//    - A. Settings and Miscellaneous -

#define        BJL_BEGIN_SETTINGS            "\033[K\02\00\00\037BJLSTART",    15,    0,    0            // A-1    ##
#define        BJL_END_SETTINGS              "BJLEND",                          6,    0,    0            // A-2    ##
#define        BJL_CONTROL_MODE              "ControlMode",                    11,    3,    0            // A-3    *1
//    #define        BJL_RESERVED_COMMAND        "Reserve",                           7,    1,    1        // A-4    Except A250
//    #define        BJL_@RESERVED_COMMAND       "@Reserve",                          8,    1,    0        // A-5    Except A250

//    - B. Execution and Maintenance -

#define        BJL_POWER_OFF                 "@PowerOff",                       9,    0,    0            // B-1    @31
#define        BJL_RESET                     "@Reset",                          6,    0,    0            // B-2    @32
#define        BJL_SWITCH_CONTROL            "@SetControlMode",                15,    3,    0            // B-3    @50, @51, @52
#define        BJL_EJECT_PAPER               "@Eject",                          6,    0,    0            // B-4    @33
//    #define        BJL_SWITCH_INTERFACES        "Interface",                        9,    3,    0        // B-5    Except A250
#define        BJL_NORMAL_CLEANING           "@Cleaning",                       9,    2,    0            // B-6    @39
//    #define        BJL_COLOR_CLEANING            "CleaningColor",                  13,    7,    0        // B-6'    Except A250
#define        BJL_CHANGE_HEAD               "@ChangeHead",                    11,    1,    0            // B-7    @38
//    #define        BJL_DOT_CLEAR                 "@DotCountClear",                 13,    7,    0        // B-7'    Except A250
//    #define        BJL_CHANGE_TANK               "@ChangeTank",                    11,    7,    0        // B-8    Except A250
#define        BJL_TEST_PRINT                "@TestPrint",                     10,    5,    0            // B-9    @60 ~ @64
#define        BJL_INITIALIZE_SETTINGS       "@Initialize",                    11,    6,    0            // B-10    @40 ~ @45
#define        BJL_CURRENT_SETTINGS          "@PrintStatusPage",               16,    1,    0            // B-11    @37
#define        BJL_DUMP_MODE                 "@Dump",                           5,    2,    0            // B-12    Except A250

//    - C. Basic Settings -

//    #define        BJL_PAPER_SOURCE            "SelectFeeder",                     12,    4,    1        // C-1    Except A250
#define        BJL_PAPER_SIZE                "PaperSelect",                    11,    4,    1            // C-2    *2
#define        BJL_PRINT_MODE                "PrintMode",                       9,    3,    1            // C-3    *3
#define        BJL_SMOOTHING                 "Smoothing",                       9,    2,    1            // C-4    *4
#define        BJL_SET_REDUCTION             "Reduction",                       9,    8,    1            // C-5    *35 A240 only
//    #define        BJL_BEEP                    "Beep",                              4,    2,    1        // C-6    Except A250
#define        BJL_AUTOMATIC_POWER_ON        "AutoPowerOn",                    11,    2,    1            // C-7    *5
#define        BJL_AUTOMATIC_POWER_OFF       "AutoPowerOff",                   12,    7,    1            // C-8    *6
//    #define        BJL_INTERFACE_TIME_OUT        "I/F-TimeOut",                    11,    4,    1        // C-9    Except A250
#define        BJL_SWITCH_BUFFERS            "I/D-Buffer",                     10,    2,    1            // C-10    *12

//    - D. BJ/LQ Mode Settings -

#define        BJL_SELECT_FONT               "Font",                            4,    5,    1            // D-1    *7
#define        BJL_HOST_LOCK_OUT             "HostLockOut",                    11,    2,    1            // D-2
#define        BJL_CODE_PAGE                 "CodePage",                        8,    6,    1            // D-3    *8
#define        BJL_PAGE_LENGTH               "PageLength",                     10,    3,    1            // D-4    *9
#define        BJL_CHARACTER_SET             "CharacterSet",                   12,    4,    1            // D-5    *10
#define        BJL_PAPER_LOAD_POSITION       "V-PrintPosition",                15,    4,    1            // D-6
#define        BJL_TEXT_SCALE                "TextScaleMode",                  13,    2,    1            // D-7    *11
#define        BJL_AUTOMATIC_LF              "AutoLF",                          6,    2,    1            // D-8    *13
//    #define        BJL_SHORT_TEAR_OFF            "ShortTearOff",                   12,    2,    1        // D-9    Except A250
//    #define        BJL_PERFORATION_SKIP        "PerforationSkip",                  15,    2,    1        // D-10    Except A250
#define        BJL_AUTOMATIC_CR              "AutoCR",                          6,    2,    1            // D-11    *14
#define        BJL_AGM                       "AGM",                             3,    2,    1            // D-12    *15
#define        BJL_INTERNATIONAL_CHAR        "International",                  13,   15,    1            // D-13    *16

///////
#define        BJL_LF                        "\012",                            1,    0,    0    // all commands' terminator

// *********************************************************************************
//        define BJL(E) commands' parameters for A250        Version 1.00    Jan.31,'95
// *********************************************************************************

#define        BJL_EQU                       " = ",                            3,0,0    // separator

#define        BJL_CONTROL_BJ                "BJ",                             2,0,0    // BJL_CONTROL_MODE, BJL_SWITCH_CONTROL
#define        BJL_CONTROL_LQ                "LQ",                             2,0,0    // BJL_CONTROL_MODE, BJL_SWITCH_CONTROL
#define        BJL_CONTROL_COMMON            "Common",                         6,0,0    // BJL_CONTROL_MODE
#define        BJL_CONTROL_AUTO              "Auto",                           4,0,0    // BJL_SWITCH_CONTROL

#define        BJL_INTERFACE_PARALLEL        "Parallel",                       8,0,0    // BJL_SWITCH_INTERFACES
#define        BJL_INTERFACE_SERIAL          "Serial",                         6,0,0    // BJL_SWITCH_INTERFACES
#define        BJL_INTERFACE_AUTO            "Auto",                           4,0,0    // BJL_SWITCH_INTERFACES

#define        BJL_1                         "1",                              1,0,0    // 
#define        BJL_2                         "2",                              1,0,0    // 
#define        BJL_6                         "6",                              1,0,0    // 

#define        BJL_3_0                       "3.0",                            3,0,0    // 
#define        BJL_5_0                       "5.0",                            3,0,0    // 
#define        BJL_8_0                       "8.0",                            3,0,0    // 
#define        BJL_8_5                       "8.5",                            3,0,0    // 
#define        BJL_10_0                      "10.0",                           4,0,0    // 
#define        BJL_12_7                      "12.7",                           4,0,0    // 

#define        BJL_10                        "10",                             2,0,0    // 
#define        BJL_11                        "11",                             2,0,0    // 
#define        BJL_12                        "12",                             2,0,0    // 

#define        BJL_20                        "20",                             2,0,0    // 
#define        BJL_22                        "22",                             2,0,0    // 
#define        BJL_30                        "30",                             2,0,0    // 
#define        BJL_60                        "60",                             2,0,0    // 

#define        BJL_437                       "437",                            3,0,0    // 
#define        BJL_850                       "850",                            3,0,0    // 
#define        BJL_852                       "852",                            3,0,0    // 
#define        BJL_855                       "855",                            3,0,0    //  SK, 5/22/96
#define        BJL_857                       "857",                            3,0,0    // 
#define        BJL_860                       "860",                            3,0,0    // 
#define        BJL_863                       "863",                            3,0,0    // 
#define        BJL_864                       "864",                            3,0,0    //  SK, 5/22/96
#define        BJL_865                       "865",                            3,0,0    // 
#define        BJL_869                       "869",                            3,0,0    //  SK, 5/22/96

#define        BJL_A                         "A",                              1,0,0    // 
#define        BJL_B                         "B",                              1,0,0    // 

#define        BJL_C                         "C",                              1,0,0    // 
#define        BJL_M                         "M",                              1,0,0    // 
#define        BJL_Y                         "Y",                              1,0,0    // 
#define        BJL_K                         "K",                              1,0,0    // 
#define        BJL_CMY                       "CMY",                            3,0,0    // 
#define        BJL_CMYK                      "CMYK",                           4,0,0    // 
#define        BJL_COLOR_K                   "ColorK",                         6,0,0    // 

#define        BJL_DEMO_PRINT                "DemoPrint",                      9,0,0    // 
#define        BJL_NOZZLE_CHECK              "NozzleCheck",                   11,0,0    // 

#define        BJL_FACTORY                   "Factory",                        7,0,0    // 
#define        BJL_TABLE1                    "TABLE1",                         6,0,0    // 
#define        BJL_TABLE2                    "TABLE2",                         6,0,0    // 
#define        BJL_TABLE3                    "TABLE3",                         6,0,0    // 
#define        BJL_TABLE4                    "TABLE4",                         6,0,0    // 
#define        BJL_TABLE5                    "TABLE5",                         6,0,0    // 

#define        BJL_ON                        "On",                             2,0,0    // 
#define        BJL_OFF                       "Off",                            3,0,0    // 

#define        BJL_BIN_1                     "Bin1",                           4,0,0    // 
#define        BJL_BIN_2                     "Bin2",                           4,0,0    // 
#define        BJL_TRACTOR                   "Tractor",                        7,0,0    // 
#define        BJL_MANUAL                    "Manual",                         6,0,0    // 

#define        BJL_A4                        "A4",                             2,0,0    // 
#define        BJL_LETTER                    "Letter",                         6,0,0    // 
#define        BJL_10_ENVELOPE               "#10 Envelope",                  12,0,0    // 
#define        BJL_DL_ENVELOPE               "DL Envelope",                   11,0,0    // 
#define        BJL_B5                        "B5",                             2,0,0    // 
#define        BJL_LEGAL                     "Legal",                          5,0,0    // 

#define        BJL_HQ                        "HQ",                             2,0,0    // 
#define        BJL_HS                        "HS",                             2,0,0    // 
#define        BJL_FINE                      "FINE",                           4,0,0    // 

#define        BJL_WIDE_1                    "Wide1",                          5,0,0    // 
#define        BJL_WIDE_2                    "Wide2",                          5,0,0    // 
#define        BJL_FORM_1                    "Form1",                          5,0,0    // 
#define        BJL_FORM_2                    "Form2",                          5,0,0    // 
#define        BJL_CUT_1                     "Cut1",                           4,0,0    // 
#define        BJL_CUT_2                     "Cut2",                           4,0,0    // 

#define        BJL_ENABLE                    "Enable",                         6,0,0    // 
#define        BJL_DISABLE                   "Disable",                        7,0,0    // 

#define        BJL_DOWNLOAD                  "Download",                       8,0,0    // 
#define        BJL_INPUT                     "Input",                          5,0,0    // 

#define        BJL_ROMAN                     "Roman",                          5,0,0    // 
#define        BJL_GOTHIC                    "Gothic",                         6,0,0    // 
#define        BJL_COURIER                   "Courier",                        7,0,0    // 
#define        BJL_PRESTIGE                  "Prestige",                       8,0,0    // 
#define        BJL_SCRIPT                    "Script",                         6,0,0    // 
#define        BJL_DRAFT                     "Draft",                          5,0,0    // 

#define        BJL_SET_1                     "Set1",                           4,0,0    // 
#define        BJL_SET_2                     "Set2",                           4,0,0    // 
#define        BJL_ITALIC                    "Italic",                         6,0,0    // 
#define        BJL_GRAPHICS                  "Graphics",                       8,0,0    // 

#define        BJL_JAPAN                     "Japan",                          5,0,0    // 
#define        BJL_NORWAY                    "Norway",                         6,0,0    // 
#define        BJL_DENMARK_2                 "Denmark2",                       8,0,0    // 
#define        BJL_SPAIN_2                   "Spain2",                         6,0,0    // 
#define        BJL_LATIN_AMERICA             "LatinAmerica",                  12,0,0    // 
#define        BJL_KOREA                     "Korea",                          5,0,0    // 
#define        BJL_LEGAL                     "Legal",                          5,0,0    // 
#define        BJL_USA                       "USA",                            3,0,0    // 
#define        BJL_FRANCE                    "France",                         6,0,0    // 
#define        BJL_GERMANY                   "Germany",                        7,0,0    // 
#define        BJL_UK                        "UK",                             2,0,0    // 
#define        BJL_DENMARK_1                 "Denmark1",                       8,0,0    // 
#define        BJL_SWEDEN                    "Sweden",                         6,0,0    // 
#define        BJL_ITALY                     "Italy",                          5,0,0    // 
#define        BJL_SPAIN_1                   "Spain1",                         6,0,0    // 
#define        BJL_SPAIN                     "Spain",                          5,0,0    // 
#define        BJL_DENMARK                   "Denmark",                        7,0,0    // 

#define        BJL_WIDE3                     "Wide3",                          5,0,0    // 
#define        BJL_FORM1                     "Form1",                          5,0,0    // 
